/**
 * Copyright (C) 2010 OSGi fuer Praktiker <info@osgi-buch.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package biz.gossipmonger.enunciator.internal;

import java.util.ArrayList;
import java.util.List;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.log.LogService;
import biz.gossipmonger.enunciator.EnunciatorService;

public class EnunciatorServiceImpl implements EnunciatorService {
  List<LogService> logServices = new ArrayList<LogService>();
  
  protected void activate(ComponentContext cc) {
    System.out.println("Enunciator activated.");
  }
  protected void deactivate(ComponentContext cc) {
    System.out.println("Enunciator deactivated.");
  }
  public void addLogService(LogService ls) {
    logServices.add(ls);
    enunciate("Enunciator got a LogService. Now there are " + logServices.size() + " references.");
  }
  public void removeLogService(LogService ls) {
    logServices.remove(ls);
    enunciate("Enunciator lost a LogService. Now there are " + logServices.size() + " references.");
  }
  public void enunciate(String text) {
    if (logServices.size() > 0) {
      for (LogService ls : logServices) {
        ls.log(LogService.LOG_INFO, text);
      }
    } else {
      System.out.println("Enunciator has no LogService; Message is: " + text);
    }
  }
}
