/**
 * Copyright (C) 2010 OSGi fuer Praktiker <info@osgi-buch.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package biz.gossipmonger.test.worker;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.ops4j.pax.exam.CoreOptions.equinox;
import static org.ops4j.pax.exam.CoreOptions.felix;
import static org.ops4j.pax.exam.CoreOptions.mavenBundle;
import static org.ops4j.pax.exam.CoreOptions.options;
import static org.ops4j.pax.exam.CoreOptions.provision;
import static org.ops4j.pax.exam.container.def.PaxRunnerOptions.logProfile;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.ops4j.pax.exam.Option;
import org.ops4j.pax.exam.junit.Configuration;
import org.ops4j.pax.exam.junit.JUnit4TestRunner;
import org.osgi.framework.BundleContext;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventConstants;
import org.osgi.service.event.EventHandler;

@RunWith(JUnit4TestRunner.class)
public class ExtendedWorkerServiceTest {
  @Configuration
  public static Option[] configure() {
    return options(
      felix(),
      equinox(),
      logProfile(),
      provision(
    	// Infrastructure bundles
        mavenBundle().groupId("org.osgi").artifactId("org.osgi.compendium"),
        mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.eventadmin"),
        mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.scr"),
        // Project bundles
        mavenBundle().groupId("biz.gossipmonger").artifactId("jobtimer").versionAsInProject(),
        mavenBundle().groupId("biz.gossipmonger").artifactId("biz.gossipmonger.gatekeeper").versionAsInProject(),
        mavenBundle().groupId("biz.gossipmonger").artifactId("biz.gossipmonger.worker-api").versionAsInProject(),
        mavenBundle().groupId("biz.gossipmonger").artifactId("biz.gossipmonger.worker-impl").versionAsInProject(),
        mavenBundle().groupId("biz.gossipmonger").artifactId("biz.gossipmonger.worker-config").versionAsInProject().noStart(),
        mavenBundle().groupId("biz.gossipmonger").artifactId("biz.gossipmonger.enunciator").versionAsInProject(),
        mavenBundle().groupId("biz.gossipmonger.com.itextpdf").artifactId("biz.gossipmonger.com.itextpdf.itext").versionAsInProject()));
  }

  String resultPath = null;

  private class TestEventHandler implements EventHandler {
    public void handleEvent(Event event) {
      System.out.println(event.toString());
      resultPath = (String) event.getProperty("output");
    }
  }

  @SuppressWarnings("unchecked")
  // Map -> Dictionary
  @Before
  public void setup(final BundleContext context) {
    File inputDir = new File("render_input");
    if (inputDir.exists()) {
      for (File f : inputDir.listFiles()) {
        f.delete();
      }
    } else {
      inputDir.mkdirs();
    }

    String[] top = new String[] { "biz/gossipmonger/gatekeeper" };
    Map<String, Object> map = new Hashtable<String, Object>();
    map.put(EventConstants.EVENT_TOPIC, top);
    context.registerService(EventHandler.class.getName(), new TestEventHandler(), (Dictionary<String, Object>) map);

    try {
      File file = new File(inputDir, "extended.txt");
      FileWriter fw = new FileWriter(file);
      fw.append("Testfile content.\n\nTwo new lines before this one.");
      fw.close();
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  @Test
  public void workerServiceResultEventOccurs(final BundleContext bundleContext) {
    synchronized (this) {
      int i = 0;
      while (i < 12 && resultPath == null) {
        try {
          System.out.println("Wait a second (" + ++i + ")");
          wait(1000);
        } catch (InterruptedException e) {
          assertTrue("Testcase was interrupted", false);
        }
      }
      if (i == 12) {
        assertTrue("Testcase has timed out", false);
      }
    }
    File file = new File(resultPath);
    assertTrue("Result path doesn't exist.", file.exists());
    assertTrue("Result path is no file.", file.isFile());
    String parentName = file.getParentFile().getName();
    assertEquals("Result path is not configured by in-bundle properties file", "render_outfragment", parentName);
  }
}
